# Copyright (c) Jupyter Development Team.
# Distributed under the terms of the Modified BSD License.
"""
Common validator wrapper to provide a uniform usage of other schema validation
libraries.
"""

import os

import jsonschema
from jsonschema import Draft4Validator as _JsonSchemaValidator
from jsonschema import ValidationError

try:
    import fastjsonschema
    from fastjsonschema import JsonSchemaException as _JsonSchemaException
except ImportError:
    fastjsonschema = None
    _JsonSchemaException = ValidationError


class JsonSchemaValidator:
    name = "jsonschema"

    def __init__(self, schema):
        self._schema = schema
        self._default_validator = _JsonSchemaValidator(schema)  # Default
        self._validator = self._default_validator

    def validate(self, data):
        self._default_validator.validate(data)

    def iter_errors(self, data, schema=None):
        return self._default_validator.iter_errors(data, schema)


class FastJsonSchemaValidator(JsonSchemaValidator):
    name = "fastjsonschema"

    def __init__(self, schema):
        super().__init__(schema)
        self._validator = fastjsonschema.compile(schema)

    def validate(self, data):
        try:
            self._validator(data)
        except _JsonSchemaException as error:
            raise ValidationError(error.message, schema_path=error.path)

    def iter_errors(self, data, schema=None):
        if schema is not None:
            return self._default_validator.iter_errors(data, schema)

        errors = []
        validate_func = self._validator
        try:
            validate_func(data)
        except _JsonSchemaException as error:
            errors = [ValidationError(error.message, schema_path=error.path)]

        return errors


_VALIDATOR_MAP = [
    ("fastjsonschema", fastjsonschema, FastJsonSchemaValidator),
    ("jsonschema", jsonschema, JsonSchemaValidator),
]
VALIDATORS = [item[0] for item in _VALIDATOR_MAP]


def _validator_for_name(validator_name):
    if validator_name not in VALIDATORS:
        raise ValueError("Invalid validator '{0}' value!\nValid values are: {1}".format(
            validator_name, VALIDATORS))

    for (name, module, validator_cls) in _VALIDATOR_MAP:
        if module and validator_name == name:
            return validator_cls


def get_current_validator():
    """
    Return the default validator based on the value of an environment variable.
    """
    validator_name = os.environ.get("NBFORMAT_VALIDATOR", "jsonschema")
    return _validator_for_name(validator_name)



"""Tests for nbformat corpus"""

# Copyright (c) Jupyter Development Team.
# Distributed under the terms of the Modified BSD License.

import pytest
import random

from .. import words

def test_acceptable_nouns_set():
    assert len(words.acceptable_nouns()) > 1000
    for word in words.acceptable_nouns():
        assert len(word) > 3, word
        assert word == word.strip()


def test_acceptable_adjectives_set():
    assert len(words.acceptable_adjectives()) > 1000
    for word in words.acceptable_adjectives():
        assert len(word) > 3, word
        assert word == word.strip()


def test_generate_corpus_id():
    assert len(words.generate_corpus_id()) > 7
    # 1 in 5073324 (3714 x 1366) times this will fail
    assert words.generate_corpus_id() !=  words.generate_corpus_id()



# -*- coding: utf-8 -*-
import os
import io
import copy

from unittest import mock
from nbformat import validate
from .. import convert
from ..nbjson import reads

from . import nbexamples
from nbformat.v3.tests import nbexamples as v3examples
from nbformat import v3, v4

def test_upgrade_notebook():
    nb03 = copy.deepcopy(v3examples.nb0)
    validate(nb03)
    nb04 = convert.upgrade(nb03)
    validate(nb04)

def test_downgrade_notebook():
    nb04 = copy.deepcopy(nbexamples.nb0)
    validate(nb04)
    nb03 = convert.downgrade(nb04)
    validate(nb03)

def test_upgrade_heading():
    # Fake the uuid generation for ids
    cell_ids = ['cell-1', 'cell-2', 'cell-3']
    with mock.patch('nbformat.v4.convert.random_cell_id', side_effect=cell_ids):
        with mock.patch('nbformat.v4.nbbase.random_cell_id', side_effect=cell_ids):
            v3h = v3.new_heading_cell
            v4m = v4.new_markdown_cell
            for v3cell, expected in [
                (
                    v3h(source='foo', level=1),
                    v4m(source='# foo'),
                ),
                (
                    v3h(source='foo\nbar\nmulti-line\n', level=4),
                    v4m(source='#### foo bar multi-line'),
                ),
                (
                    v3h(source=u'ünìcö∂e–cønvërsioñ', level=4),
                    v4m(source=u'#### ünìcö∂e–cønvërsioñ'),
                ),
            ]:
                upgraded = convert.upgrade_cell(v3cell)
                assert upgraded == expected

def test_downgrade_heading():
    v3h = v3.new_heading_cell
    v4m = v4.new_markdown_cell
    v3m = lambda source: v3.new_text_cell('markdown', source)
    for v4cell, expected in [
        (
            v4m(source='# foo'),
            v3h(source='foo', level=1),
        ),
        (
            v4m(source='#foo'),
            v3h(source='foo', level=1),
        ),
        (
            v4m(source='#\tfoo'),
            v3h(source='foo', level=1),
        ),
        (
            v4m(source='# \t  foo'),
            v3h(source='foo', level=1),
        ),
        (
            v4m(source='# foo\nbar'),
            v3m(source='# foo\nbar'),
        ),
    ]:
        downgraded = convert.downgrade_cell(v4cell)
        assert downgraded == expected

def test_upgrade_v4_to_4_dot_5():
    here = os.path.dirname(__file__)
    with io.open(os.path.join(here, os.pardir, os.pardir, 'tests', "test4.ipynb"), encoding='utf-8') as f:
        nb = reads(f.read())
    assert nb['nbformat_minor'] == 0
    validate(nb)
    assert nb.cells[0].get('id') is None

    nb_up = convert.upgrade(nb)
    assert nb_up['nbformat_minor'] == 5
    validate(nb_up)
    assert nb_up.cells[0]['id'] is not None



import os
import random

from functools import lru_cache


@lru_cache(1)
def acceptable_nouns():
    with open(os.path.join(os.path.dirname(__file__), 'nouns.txt'), 'r') as nouns_file:
        return set(word.rstrip() for word in nouns_file.readlines())


@lru_cache(1)
def acceptable_adjectives():
    with open(os.path.join(os.path.dirname(__file__), 'adjectives.txt'), 'r') as adjectives_file:
        return set(word.rstrip() for word in adjectives_file.readlines())


def generate_corpus_id():
    return '-'.join((random.sample(acceptable_adjectives(), 1)[0], random.sample(acceptable_nouns(), 1)[0]))
